@@ -336,6 +336,8 @@ init_common(int *argcp, char ***argvp, int embedded)
 			p = strrchr(app_wd, ':');
 			if ( p ) *p = 0;
 			chdir(app_wd);
+		} else {
+			fprintf(stderr, "Warning: No argv[0], cannot chdir to pythonhome\n");
 		}
 #endif
 #endif
@@ -556,13 +558,9 @@ main(int argc, char **argv)
 		if (locateResourcePy("__main__.py", scriptpath, 1024))
 			script = scriptpath;
 			
-		printf("original argc=%d\n", argc);
-		for(i=0; i<argc; i++) printf("original argv[%d] = \"%s\"\n", i, argv[i]);
 
 		init_common(&argc, &argv, 0);
 
-		printf("modified argc=%d\n", argc);
-		for(i=0; i<argc; i++) printf("modified argv[%d] = \"%s\"\n", i, argv[i]);
 	}
 
 	Py_Main(argc, argv, script);
