@@ -1,7 +1,7 @@
 /*
  /  Author: Sam Rushing <rushing@nightmare.com>
  /  Hacked for Unix by AMK
- /  $Id: mmapmodule.c,v 2.35.6.2 2002/03/08 13:40:07 mwh Exp $
+ /  $Id: mmapmodule.c,v 2.35.6.5 2003/02/07 19:46:44 nnorwitz Exp $
 
  / mmapmodule.cpp -- map a view of a file into memory
  /
@@ -141,8 +141,10 @@ mmap_close_method(mmap_object *self, PyObject *args)
 #endif /* MS_WIN32 */
 
 #ifdef UNIX
-	munmap(self->data, self->size);
-	self->data = NULL;
+	if (self->data != NULL) {
+		munmap(self->data, self->size);
+		self->data = NULL;
+	}
 #endif
 
 	Py_INCREF (Py_None);
@@ -290,7 +292,7 @@ static PyObject *
 mmap_write_method(mmap_object *self,
 		  PyObject *args)
 {
-	long length;
+	int length;
 	char *data;
 
 	CHECK_VALID(NULL);
@@ -850,6 +852,9 @@ _GetMapSize(PyObject *o)
 static PyObject *
 new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 {
+#ifdef HAVE_FSTAT
+	struct stat st;
+#endif
 	mmap_object *m_obj;
 	PyObject *map_size_obj = NULL;
 	int map_size;
@@ -890,7 +895,14 @@ new_mmap_object(PyObject *self, PyObject *args, PyObject *kwdict)
 		return PyErr_Format(PyExc_ValueError, 
 				    "mmap invalid access parameter.");
 	}
-	
+
+#ifdef HAVE_FSTAT
+	if (fstat(fd, &st) == 0 && (size_t)map_size > st.st_size) {
+		PyErr_SetString(PyExc_ValueError, 
+				"mmap length is greater than file size");
+		return NULL;
+	}
+#endif
 	m_obj = PyObject_New (mmap_object, &mmap_object_type);
 	if (m_obj == NULL) {return NULL;}
 	m_obj->size = (size_t) map_size;
